﻿using Ozeki.Network;
using System;
using System.Xml.Linq;
using Ozeki.Media;
using Ozeki.VoIP;

namespace _06_IVR_using_XML_Multi_Level
{
    class Program
    {
        public static Menu Menu = new Menu();

        static void Main(string[] args)
        {
            ShowHelp();
            
            Softphone softphone = new Softphone();

            SipAccountInitialization(softphone);

            softphone.IncomigCall += softphone_IncomigCall;

            Console.ReadLine();
        }

        private static void ShowHelp()
        {
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine("This is an example code for Interactive Voice Response (IVR) written in C#. This is a technology that allows a computer to interact with humans through the use of voice and DTMF tones input via keypad.");
            Console.WriteLine("We are going to create the IVR system by using an XML code. This makes the creation of a complex IVR so much easier.");
            Console.WriteLine("You will need to  setup your SIP account (your authentication ID, username, display name etc.). If the registration succeeded and you call the number of the IVR you will hear a greeting message.");
            Console.WriteLine("After this you can choose between two menu options.\n");

            Console.ForegroundColor = ConsoleColor.White;

        }

        static void softphone_IncomigCall(object sender, VoIPEventArgs<Ozeki.VoIP.IPhoneCall> e)
        {
            var menu = ReadXML();

            if (menu != null)
                menu.Start(e.Item);
            else
                e.Item.Reject();
        }

        private static Menu ReadXML()
        {

            string ivrXml = @"<ivr>
	                            <menu>
                                    <init>
                                        <speak>
                                               Introduce you the Interactive Voice Mail aka IVR example code written with Ozeki VoIP SDK.
                                               To get more information about Ozeki Ltd. and hear a sample mp3 song, please press button one.
                                               By pressing button two, you can listen an inform message
                                        </speak>
                                        <play>../../test.mp3</play>
                                    </init>
		                            <keys>
			                            <key pressed='1'>
                                               <speak>
                                                      Ozeki Informatics Ltd. is a leading mobile messaging software vendor and Ozeki VoIP SIP SDK is 
                                                      an excellent software development kit that allows you to establish VoIP calls from your application 
                                                      easily and quickly. You do not need to have expert programming skills, with the most basic programming 
                                                      knowledge you will be able to create extraordinary VoIP solutions with this tool.
                                               </speak>
				                               <play>../../test.mp3</play>
			                            </key>
			                            <key pressed='2'>
				                               <speak>
                                                      You pressed button two. You did nothing.
                                               </speak>
			                            </key>
                                        <key pressed='3'>
				                               <menu>
                                                      <init>
                                                            <speak>You reached the lower menu.</speak>
                                                      </init>
                                                      <keys>
                                                          <key pressed='1'>
				                                              <speak>
                                                                  You pressed button one at the lower menu level.
                                                              </speak>
			                                              </key>
                                                      </keys>
                                               </menu>
			                            </key>
		                            </keys>
	                            </menu>
                           </ivr>";

            var menu = new Menu();
            MenuLevel(ivrXml, menu);
            return menu;
        }

        private static void MenuLevel(string ivrXml, Menu menu)
        {
          try
          {
            var xelement = XElement.Parse(ivrXml);

            var menuElement = xelement.Element("menu");

            if (menuElement == null)
            {
                Console.WriteLine("Wrong XML code!");
                return;
            }

            var menuInit = menuElement.Element("init");

            foreach (var initElements in menuInit.Elements())
            {
                switch (initElements.Name.ToString())
                {
                    case "play":
                        menu.AddInitCommand(new PlayCommand(initElements.Value.ToString()));
                        break;
                    case "speak":
                        menu.AddInitCommand(new SpeakCommand(initElements.Value.ToString()));
                        break;
                }
            }

            var menuKeys = menuElement.Element("keys");

            foreach (var key in menuKeys.Elements("key"))
            {
                var pressedKeyAttribute = key.Attribute("pressed");

                if (pressedKeyAttribute == null)
                {
                    Console.WriteLine("Invalid ivr xml, keypress has no value!");
                    return;
                }

                int pressedKey;

                if (!int.TryParse(pressedKeyAttribute.Value, out pressedKey))
                {
                    Console.WriteLine("You did not add any number!");
                }

                foreach (var element in key.Elements())
                {
                    switch (element.Name.ToString())
                    {
                        case "play":
                            menu.AddKeypressCommand(pressedKey, new PlayCommand(element.Value.ToString()));
                            break;
                        case "speak":
                            menu.AddKeypressCommand(pressedKey, new SpeakCommand(element.Value.ToString()));
                            break;
                        case "menu":
                            Menu innerMenu = new Menu();
                            menu.AddKeypressCommand(pressedKey, innerMenu);
                            MenuLevel(key.ToString(), innerMenu);
                            break;
                        default:
                            return;
                    }
                }
            }
          }
          catch (Exception ex)
          {
              Console.WriteLine(ex.Message);
              Console.WriteLine("Invalid ivr xml");
          }
        }

        private static void SipAccountInitialization(Softphone softphone)
        {
            Console.WriteLine("Please setup your SIP account!\n");
            Console.WriteLine("Please set your authentication ID: ");
            var authenticationId = Read("authenticationId", true);

            Console.WriteLine("Please set your user name (default:" + authenticationId + "): ");
            var userName = Read("userName", false);
            if (string.IsNullOrEmpty(userName))
                userName = authenticationId;

            Console.WriteLine("Please set your name to be displayed (default: " + authenticationId + "): ");
            var displayName = Read("displayName", false);
            if (string.IsNullOrEmpty(displayName))
                displayName = authenticationId;

            Console.WriteLine("Please set your registration password: ");
            var registrationPassword = Read("registrationPassword", true);

            Console.WriteLine("Please set the domain name (default: your local host): ");
            var domainHost = Read("domainHost", false);
            if (string.IsNullOrEmpty(domainHost))
                domainHost = NetworkAddressHelper.GetLocalIP().ToString();

            Console.WriteLine("Please set the port number (default: 5060): ");
            int domainPort;
            string port = Read("domainPort", false);
            if (string.IsNullOrEmpty(port))
            {
                domainPort = 5060;
            }
            else
            {
                domainPort = int.Parse(port);
            }
            Console.WriteLine("\nCreating SIP account and trying to register...\n");
            softphone.Register(true, displayName, userName, authenticationId, registrationPassword, domainHost, domainPort);
        }

        private static string Read(string inputName, bool readWhileEmpty)
        {
            while (true)
            {
                string input = Console.ReadLine();

                if (!readWhileEmpty)
                {
                    return input;
                }

                if (!string.IsNullOrEmpty(input))
                {
                    return input;
                }

                Console.WriteLine(inputName + " cannot be empty!");
                Console.WriteLine(inputName + ": ");
            }
        }
    }
}
